@@ -1,4 +1,3 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/tc.str.c,v 3.47 2015/06/06 21:19:08 christos Exp $ */
 /*
  * tc.str.c: Short string package
  * 	     This has been a lesson of how to write buggy code!
@@ -36,8 +35,6 @@
 #include <assert.h>
 #include <limits.h>
 
-RCSID("$tcsh: tc.str.c,v 3.47 2015/06/06 21:19:08 christos Exp $")
-
 #define MALLOC_INCR	128
 #ifdef WIDE_STRINGS
 #define MALLOC_SURPLUS	MB_LEN_MAX /* Space for one multibyte character */
@@ -279,7 +276,7 @@ s_strncpy(Char *dst, const Char *src, size_t n)
 	return(dst);
 
     sdst = dst;
-    do 
+    do
 	if ((*dst++ = *src++) == '\0') {
 	    while (--n != 0)
 		*dst++ = '\0';
@@ -302,7 +299,7 @@ s_strncat(Char *dst, const Char *src, size_t n)
 {
     Char *sdst;
 
-    if (n == 0) 
+    if (n == 0)
 	return (dst);
 
     sdst = dst;
@@ -310,7 +307,7 @@ s_strncat(Char *dst, const Char *src, size_t n)
     while (*dst)
 	dst++;
 
-    do 
+    do
 	if ((*dst++ = *src++) == '\0')
 	    return(sdst);
     while (--n != 0)
@@ -383,7 +380,7 @@ s_strncmp(const Char *str1, const Char *str2, size_t n)
     do {
 	if (*str1 != *str2) {
 	    /*
-	     * The following case analysis is necessary so that characters 
+	     * The following case analysis is necessary so that characters
 	     * which look negative collate low against normal characters
 	     * but high against the end-of-string NUL.
 	     */
